@@ -8,7 +8,7 @@
 
 #include "config.h"
 
-#if CRYPTOPP_MSC_VERSION
+#if defined(CRYPTOPP_MSC_VERSION)
 # pragma warning(push)
 # pragma warning(disable: 4189)
 #endif
@@ -21,38 +21,38 @@ NAMESPACE_BEGIN(CryptoPP)
 
 //! \brief Inverts the cipher's direction
 //! \param dir the cipher's direction
-//! \returns DECRYPTION if dir is ENCRYPTION, DECRYPTION otherwise
+//! \returns DECRYPTION if \ref CipherDir "dir" is ENCRYPTION, DECRYPTION otherwise
 inline CipherDir ReverseCipherDir(CipherDir dir)
 {
 	return (dir == ENCRYPTION) ? DECRYPTION : ENCRYPTION;
 }
 
 //! \class FixedBlockSize
-//! \brief Inherited by block ciphers with fixed block size
-//! \tparam N the blocksize of the cipher
+//! \brief Inherited by algorithms with fixed block size
+//! \tparam N the blocksize of the algorithm
 template <unsigned int N>
 class FixedBlockSize
 {
 public:
-	//! \brief The block size of the cipher provided as a constant.
+	//! \brief The block size of the algorithm provided as a constant.
 	CRYPTOPP_CONSTANT(BLOCKSIZE = N)
 };
 
 // ************** rounds ***************
 
 //! \class FixedRounds
-//! \brief Inherited by ciphers with fixed number of rounds
-//! \tparam R the number of rounds used by the cipher
+//! \brief Inherited by algorithms with fixed number of rounds
+//! \tparam R the number of rounds used by the algorithm
 template <unsigned int R>
 class FixedRounds
 {
 public:
-	//! \brief The number of rounds for the cipher provided as a constant.
+	//! \brief The number of rounds for the algorithm provided as a constant.
 	CRYPTOPP_CONSTANT(ROUNDS = R)
 };
 
 //! \class VariableRounds
-//! \brief Inherited by ciphers with variable number of rounds
+//! \brief Inherited by algorithms with variable number of rounds
 //! \tparam D Default number of rounds
 //! \tparam N Minimum number of rounds
 //! \tparam D Maximum number of rounds
@@ -60,40 +60,54 @@ template <unsigned int D, unsigned int N=1, unsigned int M=INT_MAX>		// use INT_
 class VariableRounds
 {
 public:
-	//! \brief The default number of rounds for the cipher provided as a constant.
+	//! \brief The default number of rounds for the algorithm provided as a constant.
 	CRYPTOPP_CONSTANT(DEFAULT_ROUNDS = D)
-	//! \brief The minimum number of rounds for the cipher provided as a constant.
+	//! \brief The minimum number of rounds for the algorithm provided as a constant.
 	CRYPTOPP_CONSTANT(MIN_ROUNDS = N)
-	//! \brief The maximum number of rounds for the cipher provided as a constant.
+	//! \brief The maximum number of rounds for the algorithm provided as a constant.
 	CRYPTOPP_CONSTANT(MAX_ROUNDS = M)
-	//! \brief The default number of rounds for the cipher based on key length 
+	//! \brief The default number of rounds for the algorithm based on key length
 	//!   provided by a static function.
 	//! \param keylength the size of the key, in bytes
 	//! \details keylength is unused in the default implementation.
-	static unsigned int StaticGetDefaultRounds(size_t keylength)
-		{CRYPTOPP_UNUSED(keylength); return DEFAULT_ROUNDS;}
-
-protected:
-	//! \brief Validates the number of rounds for a cipher.
-	//! \param rounds the canddiate number of rounds
-	//! \param alg an Algorithm object used if the number of rounds are invalid
-	//! \throws InvalidRounds if the number of rounds are invalid
-	inline void ThrowIfInvalidRounds(int rounds, const Algorithm *alg)
+	CRYPTOPP_CONSTEXPR static unsigned int StaticGetDefaultRounds(size_t keylength)
 	{
-#if (M==INT_MAX)	// Coverity and result_independent_of_operands
-		if (rounds < MIN_ROUNDS)
-			throw InvalidRounds(alg ? alg->AlgorithmName() : "VariableRounds", rounds);	
+		// Comma operator breaks Debug builds with GCC 4.0 - 4.6.
+		// Also see http://github.com/weidai11/cryptopp/issues/255
+#if defined(CRYPTOPP_CXX11_CONSTEXPR)
+		return CRYPTOPP_UNUSED(keylength), static_cast<unsigned int>(DEFAULT_ROUNDS);
 #else
-		if (rounds < MIN_ROUNDS || rounds > MAX_ROUNDS)
-			throw InvalidRounds(alg ? alg->AlgorithmName() : "VariableRounds", rounds);
+		CRYPTOPP_UNUSED(keylength);
+		return static_cast<unsigned int>(DEFAULT_ROUNDS);
 #endif
 	}
 
-	//! \brief Validates the number of rounds for a cipher
-	//! \param param the canddiate number of rounds
+protected:
+	//! \brief Validates the number of rounds for an algorithm.
+	//! \param rounds the candidate number of rounds
 	//! \param alg an Algorithm object used if the number of rounds are invalid
-	//! \returns the number of rounds for the cipher
 	//! \throws InvalidRounds if the number of rounds are invalid
+	//! \details ThrowIfInvalidRounds() validates the number of rounds and throws if invalid.
+	inline void ThrowIfInvalidRounds(int rounds, const Algorithm *alg)
+	{
+		if (M == INT_MAX) // Coverity and result_independent_of_operands
+		{
+			if (rounds < MIN_ROUNDS)
+				throw InvalidRounds(alg ? alg->AlgorithmName() : std::string("VariableRounds"), rounds);
+		}
+		else
+		{
+			if (rounds < MIN_ROUNDS || rounds > MAX_ROUNDS)
+				throw InvalidRounds(alg ? alg->AlgorithmName() : std::string("VariableRounds"), rounds);
+		}
+	}
+
+	//! \brief Validates the number of rounds for an algorithm
+	//! \param param the candidate number of rounds
+	//! \param alg an Algorithm object used if the number of rounds are invalid
+	//! \returns the number of rounds for the algorithm
+	//! \throws InvalidRounds if the number of rounds are invalid
+	//! \details GetRoundsAndThrowIfInvalid() validates the number of rounds and throws if invalid.
 	inline unsigned int GetRoundsAndThrowIfInvalid(const NameValuePairs &param, const Algorithm *alg)
 	{
 		int rounds = param.GetIntValueWithDefault("Rounds", DEFAULT_ROUNDS);
@@ -107,37 +121,47 @@ protected:
 //! \class FixedKeyLength
 //! \brief Inherited by keyed algorithms with fixed key length
 //! \tparam N Default key length, in bytes
-//! \tparam IV_REQ The IV requirements. See IV_Requirement in cryptlib.h for allowed values
-//! \tparam IV_L Default IV length, in bytes
+//! \tparam IV_REQ the \ref SimpleKeyingInterface::IV_Requirement "IV requirements"
+//! \tparam IV_L default IV length, in bytes
+//! \sa SimpleKeyingInterface
 template <unsigned int N, unsigned int IV_REQ = SimpleKeyingInterface::NOT_RESYNCHRONIZABLE, unsigned int IV_L = 0>
 class FixedKeyLength
 {
 public:
-	//! \brief The default key length used by the cipher provided as a constant
+	//! \brief The default key length used by the algorithm provided as a constant
 	//! \details KEYLENGTH is provided in bytes, not bits
 	CRYPTOPP_CONSTANT(KEYLENGTH=N)
-	//! \brief The minimum key length used by the cipher provided as a constant
+	//! \brief The minimum key length used by the algorithm provided as a constant
 	//! \details MIN_KEYLENGTH is provided in bytes, not bits
 	CRYPTOPP_CONSTANT(MIN_KEYLENGTH=N)
-	//! \brief The maximum key length used by the cipher provided as a constant
+	//! \brief The maximum key length used by the algorithm provided as a constant
 	//! \details MAX_KEYLENGTH is provided in bytes, not bits
 	CRYPTOPP_CONSTANT(MAX_KEYLENGTH=N)
-	//! \brief The default key length used by the cipher provided as a constant
+	//! \brief The default key length used by the algorithm provided as a constant
 	//! \details DEFAULT_KEYLENGTH is provided in bytes, not bits
 	CRYPTOPP_CONSTANT(DEFAULT_KEYLENGTH=N)
-	//! \brief The default IV requirements for the cipher provided as a constant
+	//! \brief The default IV requirements for the algorithm provided as a constant
 	//! \details The default value is NOT_RESYNCHRONIZABLE. See IV_Requirement
 	//!  in cryptlib.h for allowed values.
 	CRYPTOPP_CONSTANT(IV_REQUIREMENT = IV_REQ)
-	//! \brief The default IV length used by the cipher provided as a constant
+	//! \brief The default IV length used by the algorithm provided as a constant
 	//! \details IV_LENGTH is provided in bytes, not bits. The default implementation uses 0.
 	CRYPTOPP_CONSTANT(IV_LENGTH = IV_L)
-	//! \brief The default key length for the cipher provided by a static function.
+	//! \brief The default key length for the algorithm provided by a static function.
 	//! \param keylength the size of the key, in bytes
 	//! \details The default implementation returns KEYLENGTH. keylength is unused
 	//!   in the default implementation.
-	static size_t CRYPTOPP_API StaticGetValidKeyLength(size_t keylength)
-		{CRYPTOPP_UNUSED(keylength); return KEYLENGTH;}
+	CRYPTOPP_CONSTEXPR static size_t CRYPTOPP_API StaticGetValidKeyLength(size_t keylength)
+	{
+		// Comma operator breaks Debug builds with GCC 4.0 - 4.6.
+		// Also see http://github.com/weidai11/cryptopp/issues/255
+#if defined(CRYPTOPP_CXX11_CONSTEXPR)
+		return CRYPTOPP_UNUSED(keylength), static_cast<size_t>(KEYLENGTH);
+#else
+		CRYPTOPP_UNUSED(keylength);
+		return static_cast<size_t>(KEYLENGTH);
+#endif
+	}
 };
 
 //! \class VariableKeyLength
@@ -146,8 +170,9 @@ public:
 //! \tparam N Minimum key length, in bytes
 //! \tparam M Maximum key length, in bytes
 //! \tparam M Default key length multiple, in bytes. The default multiple is 1.
-//! \tparam IV_REQ The IV requirements. See IV_Requirement in cryptlib.h for allowed values
-//! \tparam IV_L Default IV length, in bytes. The default length is 0.
+//! \tparam IV_REQ the \ref SimpleKeyingInterface::IV_Requirement "IV requirements"
+//! \tparam IV_L default IV length, in bytes. The default length is 0.
+//! \sa SimpleKeyingInterface
 template <unsigned int D, unsigned int N, unsigned int M, unsigned int Q = 1, unsigned int IV_REQ = SimpleKeyingInterface::NOT_RESYNCHRONIZABLE, unsigned int IV_L = 0>
 class VariableKeyLength
 {
@@ -160,26 +185,26 @@ class VariableKeyLength
 	CRYPTOPP_COMPILE_ASSERT(M >= D);
 
 public:
-	//! \brief The minimum key length used by the cipher provided as a constant
+	//! \brief The minimum key length used by the algorithm provided as a constant
 	//! \details MIN_KEYLENGTH is provided in bytes, not bits
 	CRYPTOPP_CONSTANT(MIN_KEYLENGTH=N)
-	//! \brief The maximum key length used by the cipher provided as a constant
+	//! \brief The maximum key length used by the algorithm provided as a constant
 	//! \details MAX_KEYLENGTH is provided in bytes, not bits
 	CRYPTOPP_CONSTANT(MAX_KEYLENGTH=M)
-	//! \brief The default key length used by the cipher provided as a constant
+	//! \brief The default key length used by the algorithm provided as a constant
 	//! \details DEFAULT_KEYLENGTH is provided in bytes, not bits
 	CRYPTOPP_CONSTANT(DEFAULT_KEYLENGTH=D)
-	//! \brief The key length multiple used by the cipher provided as a constant
+	//! \brief The key length multiple used by the algorithm provided as a constant
 	//! \details MAX_KEYLENGTH is provided in bytes, not bits
 	CRYPTOPP_CONSTANT(KEYLENGTH_MULTIPLE=Q)
-	//! \brief The default IV requirements for the cipher provided as a constant
+	//! \brief The default IV requirements for the algorithm provided as a constant
 	//! \details The default value is NOT_RESYNCHRONIZABLE. See IV_Requirement
 	//!  in cryptlib.h for allowed values.
 	CRYPTOPP_CONSTANT(IV_REQUIREMENT=IV_REQ)
-	//! \brief The default initialization vector length for the cipher provided as a constant
+	//! \brief The default initialization vector length for the algorithm provided as a constant
 	//! \details IV_LENGTH is provided in bytes, not bits. The default implementation uses 0.
 	CRYPTOPP_CONSTANT(IV_LENGTH=IV_L)
-	//! \brief Provides a valid key length for the cipher provided by a static function.
+	//! \brief Provides a valid key length for the algorithm provided by a static function.
 	//! \param keylength the size of the key, in bytes
 	//! \details If keylength is less than MIN_KEYLENGTH, then the function returns
 	//!   MIN_KEYLENGTH. If keylength is greater than MAX_KEYLENGTH, then the function
@@ -187,14 +212,12 @@ public:
 	//!   then keylength is returned. Otherwise, the function returns keylength rounded
 	//!   \a down to the next smaller multiple of KEYLENGTH_MULTIPLE.
 	//! \details keylength is provided in bytes, not bits.
+	// TODO: Figure out how to make this CRYPTOPP_CONSTEXPR
 	static size_t CRYPTOPP_API StaticGetValidKeyLength(size_t keylength)
 	{
-#if MIN_KEYLENGTH > 0
 		if (keylength < (size_t)MIN_KEYLENGTH)
 			return MIN_KEYLENGTH;
-		else
-#endif
-		if (keylength > (size_t)MAX_KEYLENGTH)
+		else if (keylength > (size_t)MAX_KEYLENGTH)
 			return (size_t)MAX_KEYLENGTH;
 		else
 		{
@@ -207,29 +230,30 @@ public:
 //! \class SameKeyLengthAs
 //! \brief Provides key lengths based on another class's key length
 //! \tparam T another FixedKeyLength or VariableKeyLength class
-//! \tparam IV_REQ The IV requirements. See IV_Requirement in cryptlib.h for allowed values
-//! \tparam IV_L Default IV length, in bytes
+//! \tparam IV_REQ the \ref SimpleKeyingInterface::IV_Requirement "IV requirements"
+//! \tparam IV_L default IV length, in bytes
+//! \sa SimpleKeyingInterface
 template <class T, unsigned int IV_REQ = SimpleKeyingInterface::NOT_RESYNCHRONIZABLE, unsigned int IV_L = 0>
 class SameKeyLengthAs
 {
 public:
-	//! \brief The minimum key length used by the cipher provided as a constant
+	//! \brief The minimum key length used by the algorithm provided as a constant
 	//! \details MIN_KEYLENGTH is provided in bytes, not bits
 	CRYPTOPP_CONSTANT(MIN_KEYLENGTH=T::MIN_KEYLENGTH)
-	//! \brief The maximum key length used by the cipher provided as a constant
+	//! \brief The maximum key length used by the algorithm provided as a constant
 	//! \details MIN_KEYLENGTH is provided in bytes, not bits
 	CRYPTOPP_CONSTANT(MAX_KEYLENGTH=T::MAX_KEYLENGTH)
-	//! \brief The default key length used by the cipher provided as a constant
+	//! \brief The default key length used by the algorithm provided as a constant
 	//! \details MIN_KEYLENGTH is provided in bytes, not bits
 	CRYPTOPP_CONSTANT(DEFAULT_KEYLENGTH=T::DEFAULT_KEYLENGTH)
-	//! \brief The default IV requirements for the cipher provided as a constant
+	//! \brief The default IV requirements for the algorithm provided as a constant
 	//! \details The default value is NOT_RESYNCHRONIZABLE. See IV_Requirement
 	//!  in cryptlib.h for allowed values.
 	CRYPTOPP_CONSTANT(IV_REQUIREMENT=IV_REQ)
-	//! \brief The default initialization vector length for the cipher provided as a constant
+	//! \brief The default initialization vector length for the algorithm provided as a constant
 	//! \details IV_LENGTH is provided in bytes, not bits. The default implementation uses 0.
 	CRYPTOPP_CONSTANT(IV_LENGTH=IV_L)
-	//! \brief Provides a valid key length for the cipher provided by a static function.
+	//! \brief Provides a valid key length for the algorithm provided by a static function.
 	//! \param keylength the size of the key, in bytes
 	//! \details If keylength is less than MIN_KEYLENGTH, then the function returns
 	//!   MIN_KEYLENGTH. If keylength is greater than MAX_KEYLENGTH, then the function
@@ -237,34 +261,41 @@ public:
 	//!   then keylength is returned. Otherwise, the function returns keylength rounded
 	//!   \a down to the next smaller multiple of KEYLENGTH_MULTIPLE.
 	//! \details keylength is provided in bytes, not bits.
-	static size_t CRYPTOPP_API StaticGetValidKeyLength(size_t keylength)
+	CRYPTOPP_CONSTEXPR static size_t CRYPTOPP_API StaticGetValidKeyLength(size_t keylength)
 		{return T::StaticGetValidKeyLength(keylength);}
 };
 
-// ************** implementation helper for SimpleKeyed ***************
+// ************** implementation helper for SimpleKeyingInterface ***************
 
 //! \class SimpleKeyingInterfaceImpl
-//! \brief Provides class member functions to access SimpleKeyingInterface constants
+//! \brief Provides a base implementation of SimpleKeyingInterface
 //! \tparam BASE a SimpleKeyingInterface derived class
 //! \tparam INFO a SimpleKeyingInterface derived class
+//! \details SimpleKeyingInterfaceImpl() provides a default implementation for ciphers providing a keying interface.
+//!   Functions are virtual and not eligible for C++11 <tt>constexpr</tt>-ness.
+//! \sa Algorithm(), SimpleKeyingInterface()
 template <class BASE, class INFO = BASE>
 class CRYPTOPP_NO_VTABLE SimpleKeyingInterfaceImpl : public BASE
 {
 public:
-	//! \brief The minimum key length used by the cipher
+	//! \brief The minimum key length used by the algorithm
+	//! \returns minimum key length used by the algorithm, in bytes
 	size_t MinKeyLength() const
 		{return INFO::MIN_KEYLENGTH;}
 
-	//! \brief The maximum key length used by the cipher
+	//! \brief The maximum key length used by the algorithm
+	//! \returns maximum key length used by the algorithm, in bytes
 	size_t MaxKeyLength() const
 		{return (size_t)INFO::MAX_KEYLENGTH;}
-	
-	//! \brief The default key length used by the cipher
+
+	//! \brief The default key length used by the algorithm
+	//! \returns default key length used by the algorithm, in bytes
 	size_t DefaultKeyLength() const
 		{return INFO::DEFAULT_KEYLENGTH;}
-	
-	//! \brief Provides a valid key length for the cipher
+
+	//! \brief Provides a valid key length for the algorithm
 	//! \param keylength the size of the key, in bytes
+	//! \returns the valid key lenght, in bytes
 	//! \details keylength is provided in bytes, not bits. If keylength is less than MIN_KEYLENGTH,
 	//!   then the function returns MIN_KEYLENGTH. If keylength is greater than MAX_KEYLENGTH,
 	//!   then the function returns MAX_KEYLENGTH. if If keylength is a multiple of KEYLENGTH_MULTIPLE,
@@ -272,28 +303,31 @@ public:
 	//!   KEYLENGTH_MULTIPLE.
 	size_t GetValidKeyLength(size_t keylength) const {return INFO::StaticGetValidKeyLength(keylength);}
 
-	//! \brief The default IV requirements for the cipher
+	//! \brief The default IV requirements for the algorithm
 	//! \details The default value is NOT_RESYNCHRONIZABLE. See IV_Requirement
 	//!  in cryptlib.h for allowed values.
 	SimpleKeyingInterface::IV_Requirement IVRequirement() const
 		{return (SimpleKeyingInterface::IV_Requirement)INFO::IV_REQUIREMENT;}
-	
-	//! \brief The default initialization vector length for the cipher
+
+	//! \brief The default initialization vector length for the algorithm
 	//! \details IVSize is provided in bytes, not bits. The default implementation uses IV_LENGTH, which is 0.
 	unsigned int IVSize() const
 		{return INFO::IV_LENGTH;}
 };
 
 //! \class BlockCipherImpl
-//! \brief Provides class member functions to access BlockCipher constants
+//! \brief Provides a base implementation of Algorithm and SimpleKeyingInterface for block ciphers
 //! \tparam INFO a SimpleKeyingInterface derived class
 //! \tparam BASE a SimpleKeyingInterface derived class
+//! \details BlockCipherImpl() provides a default implementation for block ciphers using AlgorithmImpl()
+//!   and SimpleKeyingInterfaceImpl(). Functions are virtual and not eligible for C++11 <tt>constexpr</tt>-ness.
+//! \sa Algorithm(), SimpleKeyingInterface(), AlgorithmImpl(), SimpleKeyingInterfaceImpl()
 template <class INFO, class BASE = BlockCipher>
 class CRYPTOPP_NO_VTABLE BlockCipherImpl : public AlgorithmImpl<SimpleKeyingInterfaceImpl<TwoBases<BASE, INFO> > >
 {
 public:
-	//! Provides the block size of the cipher
-	//! \returns the block size of the cipher, in bytes
+	//! Provides the block size of the algorithm
+	//! \returns the block size of the algorithm, in bytes
 	unsigned int BlockSize() const {return this->BLOCKSIZE;}
 };
 
@@ -315,7 +349,7 @@ public:
 	//!    SimpleKeyingInterface::SetKey.
 	BlockCipherFinal(const byte *key)
 		{this->SetKey(key, this->DEFAULT_KEYLENGTH);}
-	
+
 	//! \brief Construct a BlockCipherFinal
 	//! \param key a byte array used to key the cipher
 	//! \param length the length of the byte array
@@ -323,7 +357,7 @@ public:
 	//!    SimpleKeyingInterface::SetKey.
 	BlockCipherFinal(const byte *key, size_t length)
 		{this->SetKey(key, length);}
-	
+
 	//! \brief Construct a BlockCipherFinal
 	//! \param key a byte array used to key the cipher
 	//! \param length the length of the byte array
@@ -335,14 +369,18 @@ public:
 
 	//! \brief Provides the direction of the cipher
 	//! \returns true if DIR is ENCRYPTION, false otherwise
-	//! \sa IsForwardTransformation(), IsPermutation(), GetCipherDirection()
+	//! \sa GetCipherDirection(), IsPermutation()
 	bool IsForwardTransformation() const {return DIR == ENCRYPTION;}
 };
 
 //! \class MessageAuthenticationCodeImpl
-//! \brief Provides class member functions to access MessageAuthenticationCode constants
+//! \brief Provides a base implementation of Algorithm and SimpleKeyingInterface for message authentication codes
 //! \tparam INFO a SimpleKeyingInterface derived class
 //! \tparam BASE a SimpleKeyingInterface derived class
+//! \details MessageAuthenticationCodeImpl() provides a default implementation for message authentication codes
+//!   using AlgorithmImpl() and SimpleKeyingInterfaceImpl(). Functions are virtual and not subject to C++11
+//!   <tt>constexpr</tt>.
+//! \sa Algorithm(), SimpleKeyingInterface(), AlgorithmImpl(), SimpleKeyingInterfaceImpl()
 template <class BASE, class INFO = BASE>
 class MessageAuthenticationCodeImpl : public AlgorithmImpl<SimpleKeyingInterfaceImpl<BASE, INFO>, INFO>
 {
@@ -360,13 +398,13 @@ public:
 	//! \details The message authentication code is not keyed.
  	MessageAuthenticationCodeFinal() {}
 	//! \brief Construct a BlockCipherFinal
-	//! \param key a byte array used to key the cipher
+	//! \param key a byte array used to key the algorithm
 	//! \details key must be at least DEFAULT_KEYLENGTH in length. Internally, the function calls
 	//!    SimpleKeyingInterface::SetKey.
 	MessageAuthenticationCodeFinal(const byte *key)
 		{this->SetKey(key, this->DEFAULT_KEYLENGTH);}
 	//! \brief Construct a BlockCipherFinal
-	//! \param key a byte array used to key the cipher
+	//! \param key a byte array used to key the algorithm
 	//! \param length the length of the byte array
 	//! \details key must be at least DEFAULT_KEYLENGTH in length. Internally, the function calls
 	//!    SimpleKeyingInterface::SetKey.
@@ -380,7 +418,7 @@ public:
 //! \brief Provides Encryption and Decryption typedefs used by derived classes to
 //!    implement a block cipher
 //! \details These objects usually should not be used directly. See CipherModeDocumentation
-//!    instead. Each class derived from this one defines two types, Encryption and Decryption, 
+//!    instead. Each class derived from this one defines two types, Encryption and Decryption,
 //!    both of which implement the BlockCipher interface.
 struct BlockCipherDocumentation
 {
@@ -393,7 +431,7 @@ struct BlockCipherDocumentation
 //! \class SymmetricCipherDocumentation
 //! \brief Provides Encryption and Decryption typedefs used by derived classes to
 //!    implement a symmetric cipher
-//! \details Each class derived from this one defines two types, Encryption and Decryption, 
+//! \details Each class derived from this one defines two types, Encryption and Decryption,
 //!    both of which implement the SymmetricCipher interface. Two types of classes derive
 //!    from this class: stream ciphers and block cipher modes. Stream ciphers can be used
 //!    alone, cipher mode classes need to be used with a block cipher. See CipherModeDocumentation
@@ -409,7 +447,7 @@ struct SymmetricCipherDocumentation
 //! \class AuthenticatedSymmetricCipherDocumentation
 //! \brief Provides Encryption and Decryption typedefs used by derived classes to
 //!    implement an authenticated encryption cipher
-//! \details Each class derived from this one defines two types, Encryption and Decryption, 
+//! \details Each class derived from this one defines two types, Encryption and Decryption,
 //!    both of which implement the AuthenticatedSymmetricCipher interface.
 struct AuthenticatedSymmetricCipherDocumentation
 {
@@ -420,8 +458,8 @@ struct AuthenticatedSymmetricCipherDocumentation
 };
 
 NAMESPACE_END
-	
-#if CRYPTOPP_MSC_VERSION
+
+#if defined(CRYPTOPP_MSC_VERSION)
 # pragma warning(pop)
 #endif
 
