@@ -11,13 +11,16 @@
 
 NAMESPACE_BEGIN(CryptoPP)
 
-//! _
-struct Serpent_Info : public FixedBlockSize<16>, public VariableKeyLength<16, 0, 32>, public FixedRounds<32>
+//! \class Serpent_Info
+//! \brief Serpent block cipher information
+struct Serpent_Info : public FixedBlockSize<16>, public VariableKeyLength<16, 16, 32, 8>, public FixedRounds<32>
 {
-	static const char *StaticAlgorithmName() {return "Serpent";}
+	CRYPTOPP_CONSTEXPR static const char *StaticAlgorithmName() {return "Serpent";}
 };
 
-/// <a href="http://www.weidai.com/scan-mirror/cs.html#Serpent">Serpent</a>
+//! \class Serpent
+//! \brief Serpent block cipher
+/// \sa <a href="http://www.weidai.com/scan-mirror/cs.html#Serpent">Serpent</a>
 class Serpent : public Serpent_Info, public BlockCipherDocumentation
 {
 	class CRYPTOPP_NO_VTABLE Base : public BlockCipherImpl<Serpent_Info>
