@@ -2,11 +2,14 @@
 #define CRYPTOPP_VALIDATE_H
 
 #include "cryptlib.h"
+#include <iostream>
+#include <iomanip>
 
 bool ValidateAll(bool thorough);
 bool TestSettings();
 bool TestOS_RNG();
 bool TestAutoSeeded();
+bool TestAutoSeededX917();
 
 #if (CRYPTOPP_BOOL_X86 || CRYPTOPP_BOOL_X32 || CRYPTOPP_BOOL_X64)
 bool TestRDRAND();
@@ -15,6 +18,7 @@ bool TestRDSEED();
 
 bool ValidateBaseCode();
 bool ValidateCRC32();
+bool ValidateCRC32C();
 bool ValidateAdler32();
 bool ValidateMD2();
 bool ValidateMD4();
@@ -25,6 +29,8 @@ bool ValidateTiger();
 bool ValidateRIPEMD();
 bool ValidatePanama();
 bool ValidateWhirlpool();
+bool ValidateBLAKE2s();
+bool ValidateBLAKE2b();
 
 bool ValidateHMAC();
 bool ValidateTTMAC();
@@ -65,6 +71,8 @@ bool ValidateCMAC();
 bool ValidateBBS();
 bool ValidateDH();
 bool ValidateMQV();
+bool ValidateHMQV();
+bool ValidateFHMQV();
 bool ValidateRSA();
 bool ValidateElGamal();
 bool ValidateDLIES();
@@ -82,16 +90,41 @@ bool ValidateEC2N();
 bool ValidateECDSA();
 bool ValidateESIGN();
 
-#if !defined(NDEBUG)
+#if CRYPTOPP_DEBUG
+bool TestSecBlock();
 bool TestPolynomialMod2();
+bool TestHuffmanCodes();
 #endif
 
-// Coverity findings
+// Coverity finding
 template <class T, bool NON_NEGATIVE>
 T StringToValue(const std::string& str);
+
+// Coverity finding
 template<>
 int StringToValue<int, true>(const std::string& str);
 
+// Coverity finding
+class StreamState
+{
+public:
+	StreamState(std::ostream& out)
+		: m_out(out), m_fmt(out.flags()), m_prec(out.precision())
+	{
+	}
+
+	~StreamState()
+	{
+		m_out.precision(m_prec);
+		m_out.flags(m_fmt);
+	}
+
+private:
+	std::ostream& m_out;
+	std::ios_base::fmtflags m_fmt;
+	std::streamsize m_prec;
+};
+
 // Functions that need a RNG; uses AES inf CFB mode with Seed.
 CryptoPP::RandomNumberGenerator & GlobalRNG();
 
